@@ -8,14 +8,20 @@ Released under the GNU GPLv2, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include <sys/types.h>  // required for kvm.h
 #include <kvm.h>
-#include <sys/param.h>
 #include <osreldate.h>
+#include <stdbool.h>
 #include <sys/jail.h>
-#include <sys/uio.h>
+#include <sys/param.h>
 #include <sys/resource.h>
+#include <sys/uio.h>
+
 #include "Hashtable.h"
-#include "DragonFlyBSDProcess.h"
+#include "ProcessList.h"
+#include "UsersTable.h"
+
+#include "dragonflybsd/DragonFlyBSDProcess.h"
 
 
 typedef struct CPUData_ {
@@ -47,10 +53,12 @@ typedef struct DragonFlyBSDProcessList_ {
    Hashtable* jails;
 } DragonFlyBSDProcessList;
 
-ProcessList* ProcessList_new(UsersTable* usersTable, Hashtable* pidMatchList, uid_t userId);
+ProcessList* ProcessList_new(UsersTable* usersTable, Hashtable* dynamicMeters, Hashtable* dynamicColumns, Hashtable* pidMatchList, uid_t userId);
 
 void ProcessList_delete(ProcessList* this);
 
 void ProcessList_goThroughEntries(ProcessList* super, bool pauseProcessUpdate);
 
+bool ProcessList_isCPUonline(const ProcessList* super, unsigned int id);
+
 #endif
