@@ -17,16 +17,15 @@ in the source distribution for its full text.
 #include "zfs/ZfsArcStats.h"
 
 
-#define JAIL_ERRMSGLEN 1024
-extern char jail_errmsg[JAIL_ERRMSGLEN];
-
 typedef struct CPUData_ {
    double userPercent;
    double nicePercent;
    double systemPercent;
    double irqPercent;
-   double idlePercent;
    double systemAllPercent;
+
+   double frequency;
+   double temperature;
 } CPUData;
 
 typedef struct FreeBSDProcessList_ {
@@ -35,15 +34,11 @@ typedef struct FreeBSDProcessList_ {
 
    unsigned long long int memWire;
    unsigned long long int memActive;
-   unsigned long long int memInactive;
-   unsigned long long int memFree;
 
    ZfsArcStats zfs;
 
    CPUData* cpus;
 
-   Hashtable* ttys;
-
    unsigned long* cp_time_o;
    unsigned long* cp_time_n;
 
@@ -52,10 +47,12 @@ typedef struct FreeBSDProcessList_ {
 
 } FreeBSDProcessList;
 
-ProcessList* ProcessList_new(UsersTable* usersTable, Hashtable* pidMatchList, uid_t userId);
+ProcessList* ProcessList_new(UsersTable* usersTable, Hashtable* dynamicMeters, Hashtable* dynamicColumns, Hashtable* pidMatchList, uid_t userId);
 
 void ProcessList_delete(ProcessList* this);
 
 void ProcessList_goThroughEntries(ProcessList* super, bool pauseProcessUpdate);
 
+bool ProcessList_isCPUonline(const ProcessList* super, unsigned int id);
+
 #endif
